import requests
import click
import configparser
import base64
import time
import sys
import os
import datetime
from flask import Flask, redirect, url_for, request

def twitter_session(api_key, api_secret):
    session = requests.Session()
    secret = '{}:{}'.format(api_key, api_secret)
    secret64 = base64.b64encode(secret.encode('ascii')).decode('ascii')
 
    headers = {
        'Authorization': 'Basic {}'.format(secret64),
        'Host': 'api.twitter.com',
    }
 
    r = session.post('https://api.twitter.com/oauth2/token',
                     headers=headers,
                     data={'grant_type': 'client_credentials'})
 
    bearer_token = r.json()['access_token']
 
    def bearer_auth(req):
        req.headers['Authorization'] = 'Bearer ' + bearer_token
        return req
 
    session.auth = bearer_auth
    return session
    
app = Flask(__name__) 
 
@click.group()
def cli():
    pass

@cli.command()
@click.option('--history', default=100, help='Number of tweets visible from startup')
@click.option('--noretweets', is_flag=True, help='Disable retweets')
@click.option('--retweets-cnt',  default=0, help='Set minimum of retweets count')
@click.option('--config-file',  default='auth.cfg', type=click.Path(exists=True), prompt='Path to file containing twitter key and secret')
@click.option('--tag',  default='#python', help='Set hashtag of tweets') 
def web(history, noretweets, retweets_cnt, config_file, tag):
    @app.route('/<search>')
    def search(search):
        if search[0] != '#':
            search = '#' + search
            output = tweets(history, 0, noretweets, retweets_cnt, config_file, tag, False)
            return output
    @app.route('/')
    def redirecting():
        return redirect(url_for('search',search='Python'))
    app.run()
    
    
    
@cli.command()
@click.option('--history', default=100, help='Number of tweets visible from startup')
@click.option('--period',  default=10, help='Refresh ratio of tweets')
@click.option('--noretweets', is_flag=True, help='Disable retweets')
@click.option('--retweets-cnt',  default=0, help='Set minimum of retweets count')
@click.option('--config-file',  type=click.Path(exists=True), prompt='Path to file containing twitter key and secret')
@click.option('--tag',  default='#python', help='Set hashtag of tweets') 
def console(history, period, noretweets, retweets_cnt, config_file, tag):
    tweets(history, period, noretweets, retweets_cnt, config_file, tag, True)

def tweets(history, period, noretweets, retweets_cnt, config_file, tag, console):
    print(os.getcwd())
    config = configparser.ConfigParser()
    try:
        config.read(config_file)
    except:
        print('Wrong configure file. The file should have look like auth.cfg.example')
        sys.exit(1)
    # Define default values of variables
    not_first_time = False
    since_id = 0
    # Infinite loop with sleep inside
    while True:
        try:
            session = twitter_session(config['twitter']['key'],config['twitter']['secret'])
        except Exception as e:
            print('Wrong structure of configure file. The file should have look like auth.cfg.example' + str(e))
            sys.exit(1)
        r = session.get('https://api.twitter.com/1.1/search/tweets.json', params={'q':tag,'since_id':str(since_id)})
        counter = 0
        # Looping tweets
        # Header
        tweet_text = "<table><tr><td width='7%'><b>Avatar</b></td><td align = 'center' valign='middle'><b>Time</b></td><td width='2%'></td><td align = 'center' valign='middle'><b>Tweet</b></td><td width='2%'></td></tr>"
        for tweet in r.json()['statuses']:
            tweet_line = ''
            # Disable retweets or set retweets count
            if (('retweeted_status' not in tweet) or not noretweets) and (tweet['retweet_count'] >= retweets_cnt):
                # Display only certain number of tweets - but only in first run but save the last tweet id anyways
                if (counter < history) or not_first_time:
                    if console:
                        print('Tweet: (UTC time: ' + tweet['created_at'] + ')' + "\r\n" + tweet['text'] + "\r\n\r\n")
                    else:
                        # Clickable image
                        image = "<tr><td><a href='@" + tweet['user']['screen_name'] + "'><img src='" + tweet['user']['profile_image_url'] + "'></td><td>"
                        # Convert date from UTC
                        tweet_utc = datetime.datetime.strptime(tweet['created_at'], '%a %b %d %H:%M:%S +0000 %Y')
                        tweet_local = tweet_utc + datetime.timedelta(hours = 2)
                        # Tweet text for indicies purposes
                        text = tweet['text']
                        tweet_line = tweet_line + "</td><td>"  + tweet['text'] + "</td><td>"
                        # Hastags links
                        for hashtag in tweet['entities']['hashtags']:
                            substring_hashtag = text[hashtag['indices'][0]:hashtag['indices'][1]]
                            click_hashtag = "<a href='" + substring_hashtag.replace('#','') + "'>" + substring_hashtag + "</a>"                           
                            if click_hashtag not in tweet_line:
                                tweet_line = tweet_line.replace(substring_hashtag, click_hashtag)
                        # URLs link
                        for urls in tweet['entities']['urls']:
                            substring_urls = text[urls['indices'][0]:urls['indices'][1]]
                            click_urls = "<a href='" + substring_urls + "'>" + substring_urls + "</a>"                           
                            if click_urls not in tweet_line:
                                tweet_line = tweet_line.replace(substring_urls, click_urls)
                        # Mentions link
                        for mentions in tweet['entities']['user_mentions']:
                            substring_mention = text[mentions['indices'][0]:mentions['indices'][1]]
                            click_mention = "<a href='" + substring_mention + "'>" + substring_mention + "</a>"                            
                            if click_mention not in tweet_line:
                                tweet_line = tweet_line.replace(substring_mention, click_mention)
                        # Media link
                        if 'media' in tweet['entities']:
                            for media in tweet['entities']['media']:
                                substring_media = text[media['indices'][0]:media['indices'][1]]
                                click_media = "<a href='" + substring_media + "'><img src='" + media['media_url'] + "' height='10%' width='10%'>"  
                                if click_media not in tweet_line:
                                    tweet_line = tweet_line.replace(substring_media, click_media)
                        tweet_text = tweet_text + image + "<b>" + str(tweet_local) + "</b><td></td>" + tweet_line + "</td></tr>"
                counter = counter + 1
            # Store tweet id to prevent list on the wall    
            if since_id < tweet['id']:
                since_id = tweet['id']
        not_first_time = True
        
        if not console:
            return tweet_text + '</table>'
        time.sleep(period)        

        
def main():
    cli()